; -----------------------------------------------------------------------------
; Evalúa si se ha alcanzado el límite inferior.
; Entrada:  A -> Límite superior (TTLLLSSS).
;           HL -> Posición actual (010TTSSS LLLCCCCC).
; Salida:   Z = Se ha alcanzado.
;           NZ = No se ha alcanzado.
; Altera el valor de los registros AF y BC.
; -----------------------------------------------------------------------------
CheckBottom:
call    checkVerticalLimit	; Compara la posición actual con el límite
; Si Z o NC, ha llegado al tope, se pone Z, de lo contrario NZ
ret     c
checkBottom_bottom:
xor     a                   ; Activa Z
ret

; -----------------------------------------------------------------------------
; Evalúa si se ha alcanzado el límite superior.
; Entrada:  A -> Margen superior (TTLLLSSS).
;           HL -> Posición actual (010TTSSS LLLCCCCC).
; Salida:   Z = Se ha alcanzado.
;           NZ = No se ha alcanzado.
; Altera el valor de los registros AF y BC.
; -----------------------------------------------------------------------------
CheckTop:
call    checkVerticalLimit  ; Compara la posición actual con el límite
; Si Z o C, ha llegado al tope, se pone Z, de lo contrario NZ
jr      c, checkTop_top     ; Ha llegado al límite superior y salta
ret     nz                  ; No ha llegado al límite superior y sale
checkTop_top:
xor     a                   ; Activa Z
ret

; -----------------------------------------------------------------------------
; Evalúa si se ha alcanzado el límite vertical.
; Entrada:  A -> Límite vertical (TTLLLSSS).
;           HL -> Posición actual (010TTSSS LLLCCCCC).
; Altera el valor de los registros AF y BC.
; -----------------------------------------------------------------------------
checkVerticalLimit:
ld      b, a    ; Guarda el valor de A en B
ld      a, h    ; Carga en A el valor de H (010TTSSSS)
and     $18     ; Se queda con el tercio
rlca
rlca
rlca            ; Pone el valor del tercio en los bits 6 y 7
ld      c, a    ; Carga el valor en C
ld      a, h    ; Vuelve a cargar en A el valor de H (010TTSSSS)
and     $07     ; Se queda con el scanline
or      c       ; Añade el tercio
ld      c, a    ; Carga el valor en C
ld      a, l    ; Carga en A el valor de L (LLLCCCCC)
and     $e0     ; Se queda con la línea
rrca
rrca            ; Pone el valor de la línea en los bits 3, 4 y 5
or      c       ; Añade el tercio y el scanline. A = TTLLLSSS
cp      b       ; Lo compara con B. B = valor original de A = Límite vertical
ret

; -----------------------------------------------------------------------------
; Limpia la pantalla, tinta 7, fondo 0.
; Altera el valor de los registros AF, BC, DE y HL.
; -----------------------------------------------------------------------------
Cls:
; Limpia los píxeles de la pantalla
ld      hl, $4000   ; Carga en HL el inicio de la VideoRAM
ld      (hl), $00   ; Limpia los píxeles de esa dirección
ld      de, $4001   ; Carga en DE la siguiente posición de la VideoRAM
ld      bc, $17ff   ; 6143 repeticiones
ldir                ; Limpia todos los píxeles de la VideoRAM

; Pone la tinta en blanco y el fondo en negro
ld      hl, $5800   ; Carga en HL el inicio del área de atributos
ld      (hl), $07   ; Lo pone con la tinta en blanco y el fondo en negro
ld      de, $5801   ; Carga en DE la siguiente posición del área de atributos
ld      bc, $2ff    ; 767 repeticiones
ldir                ; Asigna el valor a toda el área de atributos

ret

; -----------------------------------------------------------------------------
; NextScan. https://wiki.speccy.org/cursos/ensamblador/gfx2_direccionamiento
; Obtiene la posición de memoria correspondiente al scanline siguiente al indicado.
; 010T TSSS LLLC CCCC
; Entrada:  HL -> scanline actual.
; Salida:   HL -> scanline siguiente.
; Altera el valor de los registros AF y HL.
; -----------------------------------------------------------------------------
NextScan:
inc     h       ; Incrementa H para incrementar el scanline
ld      a, h    ; Carga el valor en A
and     $07     ; Se queda con los bits del scanline
ret     nz      ; Si el valor no es 0, fin de la rutina  

; Calcula la siguiente línea
ld      a, l    ; Carga el valor en A
add     a, $20  ; Añade 1 a la línea (%0010 0000)
ld      l, a    ; Carga el valor en L
ret     c       ; Si hay acarreo, ha cambiado de tercio,
                ; que ya viene ajustado de arriba. Fin de la rutina

; Si llega aquí, no ha cambiado de tercio y hay que ajustar 
; ya que el primer inc h incrementó el tercio
ld      a, h    ; Carga el valor en A
sub     $08     ; Resta un tercio (%0000 1000)
ld      h, a    ; Carga el valor el H
ret

; -----------------------------------------------------------------------------
; PreviousScan. https://wiki.speccy.org/cursos/ensamblador/gfx2_direccionamiento
; Obtiene la posición de memoria correspondiente al scanline anterior al indicado.
; 010T TSSS LLLC CCCC
; Entrada:  HL -> scanline actual.	    
; Salida:   HL -> scanline anterior.
; Altera el valor de los registros AF, BC y HL.
; -----------------------------------------------------------------------------
PreviousScan:
ld      a, h    ; Carga el valor en A
dec     h       ; Decrementa H para decrementar el scanline
and     $07     ; Se queda con los bits del scanline original
ret     nz      ; Si no estaba en el 0, fin de la rutina

; Calcula la línea anterior
ld      a, l    ; Carga el valor de L en A
sub     $20     ; Resta una línea
ld      l, a    ; Carga el valor en L
ret     c       ; Si hay acarreo, fin de la rutina

; Si llega aquí, ha pasado al scanline 7 de la línea anterior
; y ha restado un tercio, que volvemos a sumar
ld      a, h    ; Carga el valor de H en A
add     a, $08  ; Vuelve a dejar el tercio como estaba
ld      h, a    ; Carga el valor en h
ret

; -----------------------------------------------------------------------------
; Imprime la línea central.
; Altera el valor de los registros AF, B y HL.
; -----------------------------------------------------------------------------
PrintLine:
ld      b, $18          ; Se imprime en las 24 líneas de pantalla
ld      hl, $4010       ; Se empieza en la línea 0, columna 16

printLine_loop:
ld      (hl), ZERO      ; En el primer scanline se imprime el byte en blanco
inc     h               ; Pasa al siguiente scanline

push    bc              ; Preserva el valor de BC para realizar el segundo bucle
ld      b, $06          ; Se imprime seis veces
printLine_loop2:
ld      (hl), LINE      ; Imprime el byte de la línea, $10, b00010000
inc     h               ; Pasa el siguiente scanline
djnz    printLine_loop2 ; Hasta que B = 0
pop     bc              ; Recupera el valor de BC
ld      (hl), ZERO      ; Imprime el último byte de la línea a 0
call    NextScan        ; Pasa al siguiente scanline
djnz    printLine_loop  ; Hasta que B = 0 = 24 líneas
ret

; -----------------------------------------------------------------------------
; Pinta la bola.
; Altera el valor de los registros AF, BC, DE y HL.
; -----------------------------------------------------------------------------
PrintBall:
ld      b, $00              ; Pone B a 0
ld      a, (ballRotation)   ; Obtiene la rotación de la bola, para averiguar qué pintar
ld      c, a                ; Carga el valor en C
cp      $00                 ; Compara el valor de la rotación con 0 para ver 
                            ; si rota a derecha o izquierda
ld      a, $00              ; Pone A = 0
jp      p, printBall_right  ; Si es positivo salta, rota a derecha

printBall_left:
; La rotación de la bola es a izquierda
ld      hl, ballLeft        ; Carga la dirección donde están los bytes de la bola
sub     c                   ; Resta de A el valor de C, rotación de la bola
add     a, a                ; Suma A + A. Cada definición de la bola son dos bytes
ld      c, a                ; Carga en valor en C
sbc     hl, bc              ; Resta a HL (dirección de los bytes de la bola)
                            ; el desplazamiento para posicionarse en los correctos
jr      printBall_continue

printBall_right:
; La rotación de la bola es a derecha
ld      hl, ballRight       ; Carga la dirección donde están los bytes de la bola
add     a, c                ; Suma en A el valor de C, rotación de la bola
add     a, a                ; Suma A + A. Cada definición de la bola son dos bytes
ld      c, a                ; Carga el valor en C
add     hl, bc              ; Suma a HL (dirección de los bytes de la bola)
                            ; el desplazamiento para posicionarse en los correctos

printBall_continue:
; Se carga en DE la dirección dónde está la definición de la bola
ex      de, hl
ld      hl, (ballPos)       ; Carga en HL la posición de la bola

; Pinta la primera línea en blanco
ld      (hl), ZERO          ; Mueve blanco a la posición de pantalla
inc     l                   ; Pasa a la siguiente columna
ld      (hl), ZERO          ; Mueve blanco a la posición de pantalla
dec     l                   ; Vuelve a la columna anterior
call    NextScan            ; Pasa al siguiente scanline

ld      b, $04              ; Pinta la definición de la bola en las siguientes 4 líneas
printBall_loop:
ld      a, (de)             ; Carga en A la definición de la bola
ld      (hl), a             ; Carga la definición de la bola a la posición de pantalla
inc     de                  ; Pasa al siguiente byte de la definición de la bola
inc     l                   ; Pasa a la siguiente columna
ld      a, (de)             ; Carga en A la definición de la bola
ld      (hl), a             ; Carga la definición de la bola a la posición de pantalla
dec     de                  ; Vuelve al primer byte de la definición de la bola
dec     l                   ; Vuelve a la columna anterior
call    NextScan            ; Pasa al siguiente scanline
djnz    printBall_loop      ; Hasta que B = 0

; Pinta la última línea en blanco
ld      (hl), ZERO          ; Mueve blanco a la posición de pantalla
inc     l                   ; Pasa a la siguiente columna
ld      (hl), ZERO          ; Mueve blanco a la posición de pantalla

ret

; -----------------------------------------------------------------------------
; Imprime la pala.
; Entrada:  HL -> Posición de la pala
; Altera el valor de los registros B y HL.
; -----------------------------------------------------------------------------
PrintPaddle:
ld      (hl), ZERO          ; Imprime el primer byte de la pala en blanco
call    NextScan            ; Pasa al siguiente scanline
ld      b, $16              ; Pinta el byte visible de la pala 22 veces
printPaddle_loop:
ld      (hl), PADDLE        ; Imprime el byte de la pala
call    NextScan            ; Pasa al siguiente scanline

djnz    printPaddle_loop    ; Hasta que B = 0

ld      (hl), ZERO          ; Imprime el último byte de la pala en blanco

ret